@@ -1,6 +1,6 @@
 // This contains the meta-type used by PyQt.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,7 +31,33 @@
 #include <QMetaObject>
 #include <QMetaType>
 #include <QPair>
+
+// Defining PYQT_QOBJECT_GUARD will enable code that tries to detect when a
+// QObject created by Qt (ie. where we don't have a generated virtual dtor) is
+// destroyed thus allowing us to raise an exception rather than crashing.
+// However, there are problems:
+//
+// 1. The obvious choice to implement this is QWeakPointer (for Qt v4.5 and
+//    later) or QPointer.  However they cannot be used across threads.  The
+//    code checks to see if QObject is in the current thread, but can't do
+//    anything about the QObject being moved to another thread later on.
+//
+// 2. A QObject dtor may invoke a Python reimplementation that causes the
+//    QObject to be wrapped.  QWeakPointer will fail an assertion if it is
+//    being created for a QObject that is being deleted (rather than the more
+//    sensible behaviour of creating a null QWeakPointer).
+//
+// For the moment we disable the code until we have a better solution.  (If you
+// connect the destroyed signal of a QObject then move the QObject to a
+// different thread, will the signal be delivered properly?)
+#if defined(PYQT_QOBJECT_GUARD)
+#include <QThread>
+#if QT_VERSION >= 0x040500
+#include <QWeakPointer>
+#else
 #include <QPointer>
+#endif
+#endif
 
 #include "qpycore_chimera.h"
 #include "qpycore_misc.h"
@@ -53,11 +74,13 @@ typedef int (*static_metacall_func)(sipTypeDef *, QMetaObject::Call, int,
 
 // Forward declarations.
 extern "C" {
-static PyObject *pyqtWrapperType_call(PyObject *type, PyObject *args,
-        PyObject *kwds);
 static int pyqtWrapperType_init(pyqtWrapperType *self, PyObject *args,
         PyObject *kwds);
+#if defined(PYQT_QOBJECT_GUARD)
+static PyObject *pyqtWrapperType_call(PyObject *type, PyObject *args,
+        PyObject *kwds);
 static void *qpointer_access_func(sipSimpleWrapper *w, AccessFuncOp op);
+#endif
 }
 
 static int create_dynamic_metaobject(pyqtWrapperType *pyqt_wt);
@@ -83,7 +106,11 @@ PyTypeObject qpycore_pyqtWrapperType_Type = {
     0,                      /* tp_as_sequence */
     0,                      /* tp_as_mapping */
     0,                      /* tp_hash */
+#if defined(PYQT_QOBJECT_GUARD)
     pyqtWrapperType_call,   /* tp_call */
+#else
+    0,                      /* tp_call */
+#endif
     0,                      /* tp_str */
     0,                      /* tp_getattro */
     0,                      /* tp_setattro */
@@ -633,6 +660,9 @@ static const QMetaObject *get_qmetaobject(pyqtWrapperType *pyqt_wt)
 }
 
 
+#if defined(PYQT_QOBJECT_GUARD)
+
+#if QT_VERSION < 0x040500
 // The guard for Qt generated QObjects.
 struct QObjectGuard
 {
@@ -641,10 +671,11 @@ struct QObjectGuard
     QPointer<QObject> guarded;
     void *unguarded;
 };
+#endif
 
 
-// Reimplemented to wrap any QObjects created internally by Qt in a QPointer so
-// that we can detect if they get destroyed.
+// Reimplemented to wrap any QObjects created internally by Qt so that we can
+// detect if they get destroyed.
 static PyObject *pyqtWrapperType_call(PyObject *type, PyObject *args, PyObject *kwds)
 {
     PyObject *self = sipWrapperType_Type->tp_call(type, args, kwds);
@@ -658,10 +689,20 @@ static PyObject *pyqtWrapperType_call(PyObject *type, PyObject *args, PyObject *
         // access function (e.g. qApp).
         if (!sipIsDerived(w) && !w->access_func)
         {
-            QObjectGuard *guard = new QObjectGuard(w->data);
+            QObject *qobj = reinterpret_cast<QObject *>(w->data);
 
-            w->data = guard;
-            w->access_func = qpointer_access_func;
+            // We can only guard objects in the same thread.
+            if (qobj->thread() == QThread::currentThread())
+            {
+#if QT_VERSION >= 0x040500
+                QWeakPointer<QObject> *guard = new QWeakPointer<QObject>(qobj);
+#else
+                QObjectGuard *guard = new QObjectGuard(w->data);
+#endif
+
+                w->data = guard;
+                w->access_func = qpointer_access_func;
+            }
         }
     }
 
@@ -669,20 +710,32 @@ static PyObject *pyqtWrapperType_call(PyObject *type, PyObject *args, PyObject *
 }
 
 
-// The access function for QPointers.
+// The access function for guarded QObject pointers.
 static void *qpointer_access_func(sipSimpleWrapper *w, AccessFuncOp op)
 {
+#if QT_VERSION >= 0x040500
+    QWeakPointer<QObject> *guard = reinterpret_cast<QWeakPointer<QObject> *>(w->data);
+#else
     QObjectGuard *guard = reinterpret_cast<QObjectGuard *>(w->data);
+#endif
     void *addr;
 
     switch (op)
     {
     case UnguardedPointer:
+#if QT_VERSION >= 0x040500
+        addr = guard->data();
+#else
         addr = guard->unguarded;
+#endif
         break;
 
     case GuardedPointer:
+#if QT_VERSION >= 0x040500
+        addr = guard->isNull() ? 0 : guard->data();
+#else
         addr = (QObject *)guard->guarded;
+#endif
         break;
 
     case ReleaseGuard:
@@ -693,3 +746,5 @@ static void *qpointer_access_func(sipSimpleWrapper *w, AccessFuncOp op)
 
     return addr;
 }
+
+#endif
